/** @format */

import { CODE_TYPES, EIGHT_HOURS, SEVEN_DAYS, SCAN_LEVEL } from './../constants'
import { services, paramsCheck } from '../decorator'
import { resultsModel } from '../sdk'
import { ActivityconfigService } from '../service'

export default class ActivityController {
  /**
   *  活动基本信息
   */
  @services([ActivityconfigService])
  async getActivityList(context: Context, {}, [activityconfigService]: [ActivityconfigService]) {
    const { openId } = context

    let result = await activityconfigService.getActivityListByOpenId(openId, context.data)
    if (result) {
      result.list = result.list.map((item: { _id: string }) => {
        return {
          ...item,
          activityId: item._id
        }
      })
      return resultsModel.success(result)
    }
    return resultsModel.error(CODE_TYPES.SYSTEM_ERROR, '查询活动列表失败')
  }

  /**
   *  活动详情
   */
  @paramsCheck(['activityId'])
  @services([ActivityconfigService])
  async getActivityDetail(context: Context, {}, [activityconfigService]: [ActivityconfigService]) {
    const { activityId } = context.data

    let result = await activityconfigService.getActivityInfoByActivityId(activityId)
    if (!result) return resultsModel.error(CODE_TYPES.PARAMS_ERROR, '当前活动不存在')

    const { prizeConfig, baseConfig, cardList } = result || {}

    // 奖品配置统一存在b_prize_config表中, 接口返回时根据需求重新组装
    const prizes = activityconfigService.formatPrizeConfig(prizeConfig)

    let _cardList1 = []
    const cardList1 = cardList.filter(el => el.level === SCAN_LEVEL.LEVEL1)
    const cardList2 = cardList.filter(el => el.level === SCAN_LEVEL.LEVEL2)
    const cardList3 = cardList.filter(el => el.level === SCAN_LEVEL.LEVEL3)
    if(cardList1 && cardList1.length > 0) _cardList1.push(cardList1)
    if(cardList2 && cardList2.length > 0) _cardList1.push(cardList2)
    if(cardList3 && cardList3.length > 0) _cardList1.push(cardList3)

    let activityInfo = {
      activityId,
      ...baseConfig,
      ...prizes,
      cardList: _cardList1
    }
    return resultsModel.success(activityInfo)
  }

  /**
   *  删除活动
   */
  @paramsCheck(['activityId'])
  @services([ActivityconfigService])
  async delActivity(context: Context, {}, [activityconfigService]: [ActivityconfigService]) {
    const { activityId } = context.data

    let result = await activityconfigService.deleteActivityByActivityId(activityId)
    const deleteSuccess: boolean = result === 0 || result
    if (deleteSuccess) {
      return resultsModel.success(true)
    } else {
      return resultsModel.error(CODE_TYPES.SYSTEM_ERROR, '删除活动失败')
    }
  }

  /**
   *  复制活动
   */
  @paramsCheck(['activityId'])
  @services([ActivityconfigService])
  async createCopyActivity(context: Context, { }, [activityconfigService]: [ActivityconfigService]) {
    const {
      activityId
    } = context.data;

    try {
      let result = await activityconfigService.getActivityInfoByActivityId(activityId);
      if (!result)
        return resultsModel.error(CODE_TYPES.SYSTEM_ERROR, "获取活动信息失败");
      const {
        _id,
        ...rest
      } = result.baseConfig;
      let prizeInfoList = result.prizeConfig.map((v) => {
        delete v._id;
        return v;
      })

      let cardList = result.cardList.map((v) => {
        delete v._id;
        return v;
      })

      let createResult = await activityconfigService.saveNewActivity({
        ...rest,
        title: rest.title + '副本',
        startTime: Date.now() + EIGHT_HOURS,
        endTime: Date.now() + EIGHT_HOURS + SEVEN_DAYS,
      },
        prizeInfoList,
        context.openId,
        cardList
      );
      if (createResult) {
        return resultsModel.success(createResult);
      } else {
        return resultsModel.error(CODE_TYPES.SYSTEM_ERROR, "复制活动失败");
      }
    } catch (e) {
      console.log(e);
      return resultsModel.error(CODE_TYPES.SYSTEM_ERROR, '复制活动失败')
    }
  }
}
